<?php
namespace FiloBlu\Flow\Model\From;

class Eavs extends AbstractFrom {

    const PRODUCT_NOT_FOUND_ERR_CODE = 1;
    const SKU_ATTRIBUTE_ERR_CODE = 2;
    const DROPDOWN_STORE_ERR_CODE = 3;
    const ENTITY_MANAGER_CLASS_PREFIX = 'FiloBlu\Flow\Model\From\Entity\\';
    const DEFAULT_ENTITY_TYPE_CODE = 'catalog_product';
    const ATTRIBUTE_ENTITY_TYPE_SUFFIX = 'attribute';
    const PRODUCT_ATTRIBUTE_ENTITY_TYPE_CODE = 'product_attribute';
    const EMAIL_FROM = "Eavs processor <magenteam@.it>";
    const EMAIL_TO = "";
    const PRODUCT_REF_ID = 0;
    const ATTRIBUTE_REF_ID = 1;
    
    protected $_dontRetryErrorCodes = true;


    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\Db $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Data $helper_flow,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->helper_flow = $helper_flow;
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    
    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init('FiloBlu\Flow\Model\ResourceModel\From\Eavs', 'meta_id');
    }

    
    public function process() {
        
        $this->_logger->info(__METHOD__);

        /* check store view */
        $storeId = $this->getStoreId();
        if ($storeId === false) {
            return $this->error(false, "Store view " . $this->getS() . " is missing");
        }

        $entityTypeCode = $this->getMetaRefId();
        if (empty($entityTypeCode) || $entityTypeCode == self::PRODUCT_REF_ID) {
            $entityTypeCode = self::DEFAULT_ENTITY_TYPE_CODE;
        } else if ($entityTypeCode == self::ATTRIBUTE_REF_ID) {
            $entityTypeCode = self::PRODUCT_ATTRIBUTE_ENTITY_TYPE_CODE;
        }

        $entityManagerClassName = $this->getEntityManagerClassName($entityTypeCode);
        try {
            $entityManager = \Magento\Framework\App\ObjectManager::getInstance()->create($entityManagerClassName,
            ['data' => [
                'entityIdentifier' => $this->getE(),
                'attributeCode' =>  $this->getA(),
                'attributeValue' => $this->getV(),
                'entityTypeCode' => $entityTypeCode,
                'storeId' => $storeId
                ]
            ]);
            $entityManager->process();
        } catch (\Exception $e) {
            $this->_logger->info($e->getMessage());
            //if(array_key_exists($e->getCode(),$this->_dontRetryErrorCodes)){
                return $this->error(false, $e->getMessage());
            //}
            return $this->error(true, $e->getMessage());
        }

        return $this;
    }
    

    protected function getEntityManagerClassName($entityTypeCode) {
        if ($entityTypeCode == self::DEFAULT_ENTITY_TYPE_CODE) {
            return self::ENTITY_MANAGER_CLASS_PREFIX . 'Product';
        } elseif (strstr($entityTypeCode, self::ATTRIBUTE_ENTITY_TYPE_SUFFIX) !== false) {
            return self::ENTITY_MANAGER_CLASS_PREFIX . 'Attribute';
        }

        return self::ENTITY_MANAGER_CLASS_PREFIX . 'AbstractEntity';
    }


    protected function getStoreId() {
        $storeView = $this->getS();
        $allStore = $this->helper_flow->getAllStoreIds();
        /* storeview is in s tag with codename */
        if (!empty($storeView)) {
            if (isset($allStore[$storeView])) {
                return $allStore[$storeView];
            }
        } else {
            return 0;//Mage_Core_Model_App::ADMIN_STORE_ID; //usually 0
        }

        return false;
    }

    
}